"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
function traverseRecursive(nodes, options) {
    var result = {};
    // 判定方法必须为方法
    if (!options.predicate) {
        throw new Error("predicate is not a function");
    }
    // 节点处理必须为方法
    if (!options.nodeHandle) {
        throw new Error("nodeHandle is not a function");
    }
    // 递归遍历所有节点
    function _traverse(node, nodeParams, options) {
        // 当前节点如果有子节点，则先将该节点推入到path路径中，遍历完成后再吐出
        var _a = options.predicate(node), done = _a.done, iterators = _a.iterators;
        // 无法继续递归，则为叶子节点
        nodeParams.isLeaf = done;
        // 处理node节点
        // 如果满足排除条件，则不进行节点处理
        if (options.exclude && options.exclude(node, nodeParams, options)) {
            node = node;
        }
        else {
            node = options.nodeHandle(node, nodeParams, options, result);
        }
        // 如果满足忽略条件，则不再进行下级遍历
        if (options.ignore && options.ignore(node, nodeParams, options))
            return;
        // 非叶子节点继续遍历
        if (!done) {
            // 将当前节点推入路径
            nodeParams.path.push(node);
            // 子节点进行递归
            iterators.forEach(function (subNode, index) {
                _traverse(subNode, {
                    path: nodeParams.path,
                    isFirst: index === 0,
                    isLast: index === iterators.length - 1,
                    isLeaf: false
                }, options);
            });
            // 递归完毕后，当前节点从path数组中冒泡
            nodeParams.path.pop();
        }
    }
    // 遍历首层节点
    nodes.forEach(function (subNode, index) {
        _traverse(subNode, {
            path: [],
            isFirst: index === 0,
            isLast: index === nodes.length,
            isLeaf: false
        }, options);
    });
    return result;
}
exports.traverseRecursive = traverseRecursive;
